@@ -193,9 +193,6 @@ static void     prefs_xnest_check_toggled_cb  (GtkToggleButton *button,
 					       gpointer         data);
 static void     prefs_lock_check_toggled_cb   (GtkToggleButton *button,
 					       gpointer         data);
-static void     about_url_hook                (GtkAboutDialog  *dialog,
-					       const gchar     *url,
-					       gpointer         data);
 static gboolean menubar_button_press_event_cb (GtkWidget       *menubar,
 					       GdkEventButton  *event,
 					       gpointer         data);
@@ -205,6 +202,8 @@ static gboolean menubar_expose_event_cb       (GtkWidget       *menubar,
 static void     menu_style_set_cb             (GtkWidget       *menu,
 					       GtkStyle        *old_style,
 					       gpointer         data);
+static gboolean menu_expose_cb                (GtkWidget       *menu,
+                                               gpointer         data);
 static void     menuitem_destroy_cb           (GtkWidget       *item,
 					       gpointer         data);
 static void     menuitem_style_set_cb         (GtkWidget       *menuitem,
@@ -253,7 +252,6 @@ static void      fusa_applet_data_free        (FusaAppletData *adata);
  * ****************** */
 
 static GtkTooltips *tooltips = NULL;
-static GtkWidget *about = NULL;
 static GnomeProgram *program = NULL;
 
 
@@ -485,10 +483,20 @@ applet_fill_cb (PanelApplet *applet,
   gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), adata->menu);
   g_signal_connect (adata->menu, "style-set",
 		    G_CALLBACK (menu_style_set_cb), adata);
+  g_signal_connect (adata->menu, "show",
+		    G_CALLBACK (menu_expose_cb), adata);
   gtk_widget_show (adata->menu);
 
   active_only = gconf_client_get_bool (adata->client, ACTIVE_ONLY_KEY, NULL);
 
+  /* This next part populates the list with all the users we currently know
+   * about. For almost all cases, this is the empty list, because we're
+   * asynchronous, and the data hasn't come back from the callback saying who
+   * the users are yet. However, if someone has two FUSAs on their toolbars (why,
+   * I have no freaking idea, but bear with me here), the menu of the second
+   * one to be initialised needs to be filled in from the start rather than
+   * depending on getting data from the callback like the first one.
+   */
   users = fusa_manager_list_users (adata->manager);
   while (users)
     {
@@ -521,6 +529,7 @@ applet_fill_cb (PanelApplet *applet,
 
       users = g_slist_delete_link (users, users);
     }
+
   g_signal_connect (adata->manager, "user-added",
 		    G_CALLBACK (manager_user_added_cb), adata);
 
@@ -1119,13 +1128,14 @@ about_cb (BonoboUIComponent *ui_container,
 {
   static const gchar *authors[] = {
     "James M. Cape <jcape@ignore-your.tv>",
+    "Thomas Thurman <thomas@thurman.org.uk>",
     NULL
   };
   static const gchar *artists[] = {
     "Hopefully, the users themselves",
     NULL
   };
-  static const gchar *license[] = {
+  static gchar *license[] = {
     N_("The Fast User Switch Applet is free software; you can redistribute it and/or modify "
        "it under the terms of the GNU General Public License as published by "
        "the Free Software Foundation; either version 2 of the License, or "
@@ -1141,49 +1151,25 @@ about_cb (BonoboUIComponent *ui_container,
   };
   gchar *license_i18n;
 
-  if (about)
-    {
-      gtk_window_present (GTK_WINDOW (about));
-      return;
-    }
-
-  about = gtk_about_dialog_new ();
-  g_object_add_weak_pointer (G_OBJECT (about), (gpointer *) &about);
-  gtk_about_dialog_set_logo_icon_name (GTK_ABOUT_DIALOG (about),
-				       "stock_people");
-  gtk_about_dialog_set_name (GTK_ABOUT_DIALOG (about),
-			     "Fast User Switch Applet");
-  gtk_about_dialog_set_version (GTK_ABOUT_DIALOG (about), VERSION);
-  gtk_about_dialog_set_comments (GTK_ABOUT_DIALOG (about),
-				 _("A menu to quickly switch between users."));
-  gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG (about),
-				  "Copyright \xc2\xa9 2004-2005 James M. Cape.\n"
-                                  "Copyright \xc2\xa9 2006 Thomas Thurman.");
-
-  gtk_about_dialog_set_url_hook (about_url_hook, NULL, NULL);
-  gtk_about_dialog_set_website (GTK_ABOUT_DIALOG (about),
-				"http://ignore-your.tv/fusa/");
-
-  gtk_about_dialog_set_authors (GTK_ABOUT_DIALOG (about), authors);
-  gtk_about_dialog_set_artists (GTK_ABOUT_DIALOG (about), artists);
-  /* Translators: This is a special message that shouldn't be translated
-   * literally. It is used in the about box to give credits to
-   * the translators.
-   * Thus, you should translate it to your name and email address.
-   * You should also include other translators who have contributed to
-   * this translation; in that case, please write each of them on a separate
-   * line separated by newlines (\n).
-   */
-  gtk_about_dialog_set_translator_credits (GTK_ABOUT_DIALOG (about),
-					   _("translator-credits"));
   license_i18n = g_strjoinv ("\n\n", license);
-  gtk_about_dialog_set_license (GTK_ABOUT_DIALOG (about), license_i18n);
+
+  gtk_show_about_dialog(NULL,
+      "name", _("Fast User Switch Applet"),
+      "version", VERSION,
+      "copyright", "Copyright \xc2\xa9 2004-2005 James M. Cape.\n"
+                   "Copyright \xc2\xa9 2006 Thomas Thurman.",
+      "comments", _("A menu to quickly switch between users."),
+      "authors", authors,
+      "artists", artists,
+      "license", license_i18n,
+      "wrap-license", TRUE,
+      "translator-credits", _("translator-credits"),
+      "logo-icon-name", "stock_people",
+      "website", "http://ignore-your.tv/fusa/",
+      NULL
+   );
+
   g_free (license_i18n);
-
-  g_object_add_weak_pointer (G_OBJECT (about),
-			     (gpointer *) &about);
-
-  gtk_window_present (GTK_WINDOW (about));
 }
 
 
@@ -1557,14 +1543,6 @@ prefs_lock_check_toggled_cb (GtkToggleButton *button,
 			       gtk_toggle_button_get_active (button), NULL);
 }
 
-static void
-about_url_hook (GtkAboutDialog *dialog,
-		const gchar    *url,
-		gpointer        data)
-{
-  gnome_vfs_url_show (url);
-}
-
 /*
  * gnome-panel/applets/wncklet/window-menu.c:window_filter_button_press()
  * 
@@ -1622,6 +1600,45 @@ menubar_expose_event_cb (GtkWidget      *widget,
   return FALSE;
 }
 
+/* When the user opens the menu of users, menu_expose_cb gets called.
+ * That then calls GDM to ask it which users have current logins,
+ * and when it has responded and we've updated the relevant data
+ * structures, it calls this callback.
+ *
+ * All we have to do here is repaint the menu, since the menu items
+ * are clever and will add or remove their check marks according to
+ * whether their users have any current logins.
+ */
+static void
+redraw_menu_cb (FusaManager  *manager,
+    FusaDisplay  *display,
+    const GError *error,
+    gpointer      data)
+{
+  FusaAppletData *adata = data;
+      
+  gtk_widget_queue_draw (adata->menu);
+}
+
+
+/* Called every time the menu is displayed (and also for some reason
+ * immediately it's created, which does no harm). All we have to do
+ * here is kick off a request to GDM to let us know which users are
+ * logged in, so we can display check marks next to their names.
+ */
+static gboolean
+menu_expose_cb (GtkWidget *menu,
+	          gpointer   data)
+{
+  FusaAppletData *adata = data;
+
+  fusa_manager_request_update_displays (adata->manager,
+      redraw_menu_cb,
+      data);
+
+  return FALSE;
+}
+
 static void
 menu_style_set_cb (GtkWidget *menu,
 		   GtkStyle  *old_style,
@@ -2177,7 +2194,7 @@ static void
 sort_menu (FusaAppletData *adata)
 {
   GSList *items;
-  guint n_items, n_rows, n_cols, row, column;
+  guint n_items, n_rows, n_cols, row, column, count;
   gint screen_height;
 
   if (!gtk_widget_has_screen (adata->menu))
@@ -2205,6 +2222,7 @@ sort_menu (FusaAppletData *adata)
 
   row = 0;
   column = 0;
+  count = 0;
   items = adata->items;
   while (items)
     {
@@ -2218,6 +2236,13 @@ sort_menu (FusaAppletData *adata)
 	      row = 0;
 	      column++;
 	    }
+
+          /* Just re-attaching them doesn't alter the order you get them
+           * in when you move through with the arrow keys, though; we
+           * have to set that explicitly.
+           */
+          gtk_menu_reorder_child (GTK_MENU (adata->menu),
+              items->data, count++);
 	}
 
       items = items->next;
